@@ -9,10 +9,10 @@ has keepalive => (
 );
 
 before finalize => sub {
-    my($self, $req, $res) = @_;
+    my($self, $c) = @_;
 
-    $res->headers->date(time);
-    $res->headers->header(
+    $c->res->headers->date(time);
+    $c->res->headers->header(
         Connection => $self->keepalive ? 'keep-alive' : 'close'
     );
 };
