@@ -7,7 +7,8 @@ use URI::WithBase;
 use constant should_write_response_line => 0;
 
 sub run {
-    my ( $self, $request, %args ) = @_;
+    my ( $self, $request, $env ) = @_;
+    $env ||= \%ENV;
 
     $self->handle_request(
         request_args => {
@@ -31,7 +32,6 @@ sub run {
         },
         response_args => {
         },
-        %args,
     );
 
     $self->response_writer->get_response; # FIXME yuck, should be a ret from handle_request
@@ -57,8 +57,8 @@ HTTP::Engine::Interface::Test - HTTP::Engine Test Interface
           module => 'Test',
       },
       request_handler => sub {
-          my $req = shift;
-          HTTP::Engine::Response->new( body => Dumper($req) );
+          my $c = shift;
+          $c->res->body( Dumper($c) );
       }
   )->run(HTTP::Request->new( GET => 'http://localhost/'), \%ENV);
 
