@@ -11,9 +11,10 @@ has '_response' => (
 );
 
 sub finalize {
-    my ( $self, $req, $res ) = @_;
+    my ( $self, $c ) = @_;
 
-    $self->_response($res->as_http_response);
+    $c->res->finalize($c);
+    $self->_response($c->res->as_http_response);
 }
 
 sub get_response {
