@@ -7,6 +7,7 @@ use HTTP::Body;
 use HTTP::Engine::Types::Core qw( Uri Header );
 use HTTP::Request;
 use URI::QueryParam;
+use URI::QueryParam;
 
 # Moose role merging is borked with attributes
 #with qw(HTTP::Engine::Request);
@@ -70,6 +71,12 @@ sub _build_connection_info {
     $self->request_builder->_build_connection_info($self);
 }
 
+has context => (
+    is       => 'rw',
+    isa      => 'HTTP::Engine::Context',
+    weak_ref => 1,
+);
+
 has cookies => (
     is      => 'rw',
     isa     => 'HashRef',
@@ -169,7 +176,7 @@ has hostname => (
 
 sub _build_hostname {
     my $self = shift;
-    $self->request_builder->_build_hostname($self);
+    $self->request_builder->_build_hostname;
 }
 
 has http_body => (
@@ -219,14 +226,9 @@ sub _build_parameters {
 has uploads => (
     is      => 'rw',
     isa     => 'HashRef',
-    lazy_build => 1,
+    default => sub { +{} },
 );
 
-sub _build_uploads {
-    my $self = shift;
-    $self->request_builder->_prepare_uploads($self);
-}
-
 no Moose;
 
 # aliases
@@ -246,7 +248,6 @@ sub cookie {
         return undef unless exists $self->cookies->{$name}; ## no critic.
         return $self->cookies->{$name};
     }
-    return;
 }
 
 sub param {
@@ -267,7 +268,7 @@ sub param {
               ? ( $self->parameters->{$param} )
                   : $self->parameters->{$param};
         }
-    } else {
+    } elsif (@_ > 1) {
         my $field = shift;
         $self->parameters->{$field} = [@_];
     }
@@ -291,7 +292,9 @@ sub upload {
               ? ( $self->uploads->{$upload} )
           : $self->uploads->{$upload};
         }
-    } else {
+    }
+
+    if (@_ > 1) {
         while ( my($field, $upload) = splice(@_, 0, 2) ) {
             if ( exists $self->uploads->{$field} ) {
                 for ( $self->uploads->{$field} ) {
@@ -374,8 +377,7 @@ HTTP::Engine::Request - http request object
 
 =head1 SYNOPSIS
 
-    sub handle_request {
-        my $req = shift;
+    $c->req
 
 =head1 ATTRIBUTES
 
@@ -385,6 +387,10 @@ HTTP::Engine::Request - http request object
 
 Returns the IP address of the client.
 
+=item context
+
+Returns the HTTP::Context(internal use only)
+
 =item cookies
 
 Returns a reference to a hash containing the cookies
@@ -473,22 +479,22 @@ Shortcut to $req->headers->user_agent.
 
 A convenient method to access $req->cookies.
 
-    $cookie  = $req->cookie('name');
-    @cookies = $req->cookie;
+    $cookie  = $c->req->cookie('name');
+    @cookies = $c->req->cookie;
 
 =item param
 
 Returns GET and POST parameters with a CGI.pm-compatible param method. This 
-is an alternative method for accessing parameters in $req->parameters.
+is an alternative method for accessing parameters in $c->req->parameters.
 
-    $value  = $req->param( 'foo' );
-    @values = $req->param( 'foo' );
-    @params = $req->param;
+    $value  = $c->req->param( 'foo' );
+    @values = $c->req->param( 'foo' );
+    @params = $c->req->param;
 
 Like L<CGI>, and B<unlike> earlier versions of Catalyst, passing multiple
 arguments to this method, like this:
 
-    $req->param( 'foo', 'bar', 'gorch', 'quxx' );
+    $c->req->param( 'foo', 'bar', 'gorch', 'quxx' );
 
 will set the parameter C<foo> to the multiple values C<bar>, C<gorch> and
 C<quxx>. Previously this would have added C<bar> as another value to C<foo>
@@ -503,11 +509,11 @@ Returns the path, i.e. the part of the URI after $req->base, for the current req
 
 A convenient method to access $req->uploads.
 
-    $upload  = $req->upload('field');
-    @uploads = $req->upload('field');
-    @fields  = $req->upload;
+    $upload  = $c->req->upload('field');
+    @uploads = $c->req->upload('field');
+    @fields  = $c->req->upload;
 
-    for my $upload ( $req->upload('field') ) {
+    for my $upload ( $c->req->upload('field') ) {
         print $upload->filename;
     }
 
