@@ -4,6 +4,7 @@ use Moose;
 use HTTP::Status ();
 use HTTP::Headers;
 use HTTP::Engine::Types::Core qw( Header );
+use File::stat;
 
 # Moose role merging is borked with attributes
 #with qw(HTTP::Engine::Response);
@@ -14,6 +15,11 @@ has body => (
     default => '',
 );
 
+has context => (
+    is  => 'rw',
+    isa => 'HTTP::Engine::Context',
+);
+
 has cookies => (
     is      => 'rw',
     isa     => 'HashRef',
@@ -25,6 +31,11 @@ has protocol => (
 #    isa     => 'Str',
 );
 
+has location => (
+    is  => 'rw',
+    isa => 'Str',
+);
+
 has status => (
     is      => 'rw',
     isa     => 'Int',
@@ -37,7 +48,6 @@ has headers => (
     default => sub { HTTP::Headers->new },
     handles => [ qw(content_encoding content_length content_type header) ],
 );
-no Moose;
 
 sub is_info     { HTTP::Status::is_info     (shift->status) }
 sub is_success  { HTTP::Status::is_success  (shift->status) }
@@ -46,6 +56,17 @@ sub is_error    { HTTP::Status::is_error    (shift->status) }
 
 *output = \&body;
 
+sub redirect {
+    my $self = shift;
+
+    if (@_) {
+        $self->location( shift );
+        $self->status( shift || 302 );
+    }
+
+    $self->location;
+}
+
 sub set_http_response {
     my ($self, $res) = @_;
     $self->status( $res->code );
@@ -54,6 +75,67 @@ sub set_http_response {
     $self;
 }
 
+sub finalize {
+    my ($self, $c) = @_;
+    confess 'argument missing: $c' unless $c;
+
+    # Handle redirects
+    if (my $location = $self->location ) {
+        $self->header( Location => $c->req->absolute_url($location) );
+        $self->body($self->status . ': Redirect') unless $self->body;
+    }
+
+    # Content-Length
+    $self->content_length(0);
+    if ($self->body) {
+        # get the length from a filehandle
+        if (Scalar::Util::blessed($self->body) && $self->body->can('read') or ref($self->body) eq 'GLOB') {
+            if (my $stat = stat $self->body) {
+                $self->content_length($stat->size);
+            } else {
+                warn 'Serving filehandle without a content-length';
+            }
+        } else {
+            $self->content_length(bytes::length($self->body));
+        }
+    }
+
+    # Errors
+    if ($self->status =~ /^(1\d\d|[23]04)$/) {
+        $self->headers->remove_header("Content-Length");
+        $self->body('');
+    }
+
+    $self->content_type('text/html') unless $self->content_type;
+    $self->header(Status => $self->status);
+
+    $self->_finalize_cookies();
+
+    $self->body('') if $c->req->method eq 'HEAD';
+}
+
+sub _finalize_cookies  {
+    my $self = shift;
+
+    for my $name (keys %{ $self->cookies }) {
+        my $val = $self->cookies->{$name};
+        my $cookie = (
+            Scalar::Util::blessed($val)
+            ? $val
+            : CGI::Simple::Cookie->new(
+                -name    => $name,
+                -value   => $val->{value},
+                -expires => $val->{expires},
+                -domain  => $val->{domain},
+                -path    => $val->{path},
+                -secure  => $val->{secure} || 0
+            )
+        );
+
+        $self->headers->push_header('Set-Cookie' => $cookie->as_string);
+    }
+}
+
 sub as_http_response {
     my $self = shift;
 
@@ -79,12 +161,7 @@ HTTP::Engine::Response - HTTP response object
 
 =head1 SYNOPSIS
 
-    sub handle_request {
-        my $req = shift;
-        my $res = HTTP::Engine::Response->new;
-        $res->body('foo');
-        return $res;
-    }
+    $c->res
 
 =head1 ATTRIBUTES
 
@@ -103,7 +180,7 @@ Returns a reference to a hash containing cookies to be set. The keys of the
 hash are the cookies' names, and their corresponding values are hash
 references used to construct a L<CGI::Cookie> object.
 
-        $res->cookies->{foo} = { value => '123' };
+        $c->res->cookies->{foo} = { value => '123' };
 
 The keys of the hash reference on the right correspond to the L<CGI::Cookie>
 parameters of the same name, except they are used without a leading dash.
@@ -113,20 +190,20 @@ Possible parameters are:
 
 Sets or returns the HTTP status.
 
-    $res->status(404);
+    $c->res->status(404);
 
 =item headers
 
 Returns an L<HTTP::Headers> object, which can be used to set headers.
 
-    $res->headers->header( 'X-HTTP-Engine' => $HTTP::Engine::VERSION );
+    $c->res->headers->header( 'X-HTTP-Engine' => $HTTP::Engine::VERSION );
 
 =item redirect
 
 Causes the response to redirect to the specified URL.
 
-    $res->redirect( 'http://slashdot.org' );
-    $res->redirect( 'http://slashdot.org', 307 );
+    $c->res->redirect( 'http://slashdot.org' );
+    $c->res->redirect( 'http://slashdot.org', 307 );
 
 =item set_http_response
 
