@@ -3,7 +3,6 @@ use Moose;
 use File::stat;
 use Carp;
 use HTTP::Status ();
-use HTTP::Engine::ResponseFinalizer;
 
 with qw(HTTP::Engine::Role::ResponseWriter);
 
@@ -25,21 +24,20 @@ no Moose;
 my $CRLF = "\015\012";
 
 sub finalize {
-    my($self, $req, $res) = @_;
-    Carp::croak "argument missing" unless $res;
+    my($self, $c) = @_;
+
+    local *STDOUT = $c->req->_connection->{output_handle};
+    croak "argument missing" unless $c;
 
     delete $self->{_prepared_write};
 
-    # HTTP/1.1's default Connection: close
-    if ($res->protocol && $res->protocol =~ m!1.1! && !!!$res->header('Connection')) {
-        $res->header( Connection => 'close' );
-    }
+    $c->res->protocol( $c->req->protocol ) unless $c->res->protocol;
+    $c->res->finalize($c);
 
-    local *STDOUT = $req->_connection->{output_handle};
-    $self->_write($self->_response_line($res) . $CRLF) if $self->should_write_response_line;
-    $self->_write($res->headers->as_string($CRLF));
+    $self->_write($self->_response_line($c) . $CRLF) if $self->should_write_response_line;
+    $self->_write($c->res->headers->as_string($CRLF));
     $self->_write($CRLF);
-    $self->_output_body($res);
+    $self->_output_body($c->res);
 }
 
 sub _output_body  {
@@ -47,7 +45,7 @@ sub _output_body  {
     my $body = $res->body;
 
     no warnings 'uninitialized';
-    if ((Scalar::Util::blessed($body) && $body->can('read')) || (ref($body) eq 'GLOB')) {
+    if (Scalar::Util::blessed($body) && $body->can('read') or ref($body) eq 'GLOB') {
         while (!eof $body) {
             read $body, my ($buffer), $self->chunk_size;
             last unless $self->_write($buffer);
@@ -59,9 +57,9 @@ sub _output_body  {
 }
 
 sub _response_line {
-    my ( $self, $res ) = @_;
+    my ( $self, $c ) = @_;
 
-    join(" ", $res->protocol, $res->status, HTTP::Status::status_message($res->status));
+    join(" ", $c->res->protocol, $c->res->status, HTTP::Status::status_message($c->res->status));
 }
 
 sub _write {
@@ -79,10 +77,9 @@ sub _prepare_write {
     my $self = shift;
 
     # Set the output handle to autoflush
-#    if (blessed *STDOUT) {
-    local $@;
-    eval { *STDOUT->autoflush(1); };
-#    }
+    if (blessed *STDOUT) {
+        *STDOUT->autoflush(1);
+    }
 }
 
 1;
