@@ -18,7 +18,6 @@ has chunk_size => (
     default => 4096,
 );
 
-use HTTP::Body 1.04;
 
 sub _build_http_body {
     my ( $self, $req ) = @_;
@@ -44,7 +43,7 @@ sub _build_read_state {
     my $type   = $req->header('Content-Type');
 
     my $body = HTTP::Body->new($type, $length);
-    $body->tmpdir( $self->upload_tmp) if $self->upload_tmp;
+    $body->{tmpdir} = $self->upload_tmp if $self->upload_tmp;
 
     return $self->_read_init({
         input_handle   => $req->_connection->{input_handle},
@@ -67,10 +66,10 @@ sub _handle_read_chunk {
 }
 
 sub _prepare_uploads  {
-    my($self, $req) = @_;
+    my($self, $c) = @_;
 
+    my $req     = $c->req;
     my $uploads = $req->http_body->upload;
-    my %uploads;
     for my $name (keys %{ $uploads }) {
         my $files = $uploads->{$name};
         $files = ref $files eq 'ARRAY' ? $files : [$files];
@@ -85,13 +84,12 @@ sub _prepare_uploads  {
             $u->filename($upload->{filename});
             push @uploads, $u;
         }
-        $uploads{$name} = @uploads > 1 ? \@uploads : $uploads[0];
+        $req->uploads->{$name} = @uploads > 1 ? \@uploads : $uploads[0];
 
         # support access to the filename as a normal param
         my @filenames = map { $_->{filename} } @uploads;
         $req->parameters->{$name} =  @filenames > 1 ? \@filenames : $filenames[0];
     }
-    return \%uploads;
 }
 
 __PACKAGE__
