@@ -30,17 +30,7 @@ coerce Interface, from HashRef => via {
 
 class_type Uri, { class => "URI::WithBase" };
 
-coerce Uri, from Str => via { 
-    # generate base uri
-    my $uri = URI->new($_);
-    my $base = $uri->path;
-    $base =~ s{^/+}{};
-    $uri->path($base);
-    $base .= '/' unless $base =~ /\/$/;
-    $uri->query(undef);
-    $uri->path($base);
-    URI::WithBase->new($_, $uri);
-};
+coerce Uri, from Str => via { URI::WithBase->new($_) };
 
 class_type Header, { class => "HTTP::Headers" };
 
